#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

GtkWidget *savedialog;
GtkTreeIter selection_iter;
gchar *colorname;
gchar *colorvalue = "#FFFFFF";	/* gtk colorselector defaults to white */

gchar* get_filename();


void
show_file_error                        (gchar* message)
{
	GtkDialog *d = GTK_DIALOG (
		gtk_message_dialog_new (GTK_WINDOW (gcolor2), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
			message, get_filename ())
	);

	gtk_dialog_run (d);
	gtk_widget_destroy (GTK_WIDGET (d));
}


void
on_about_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *aboutdialog;
	GtkWidget *label;
	GtkWidget *textview;
	GtkTextBuffer *textbuffer;
	GtkTextIter start, end;
	GtkTextTag *tag;
	gchar *labeltext;
	
	
	aboutdialog = create_about_dialog ();
	gtk_window_set_transient_for (GTK_WINDOW (aboutdialog), GTK_WINDOW (gcolor2));
	labeltext = g_strdup_printf (_("<span weight=\"bold\" size=\"larger\">gcolor2 v%s</span>"), VERSION);
	label = lookup_widget (aboutdialog, "about_label");
	gtk_label_set_markup (GTK_LABEL (label), labeltext);
	g_free(labeltext);
	
	textview = lookup_widget (aboutdialog, "textview1");
	textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
	gtk_text_buffer_set_text (textbuffer,
		"Developer:\n"
		"Justin Spadea <justin@j-z-s.com>\n\n"
		"Special thanks to:\n"
		"Eugenia Loli-Queru <eloli@hotmail.com>\n"
		"\t- insight on the UI design\n"
		"\t- provided the icon\n"
		"Ricardo Veguilla <veguilla@hpcf.upr.edu>\n"
		"\t- provided spec file\n\n"
		"Official Website:\n"
		"http://gcolor2.sourceforge.net"
		, -1);
	
	/* make main sections bold */
	tag = gtk_text_buffer_create_tag (textbuffer, "bold", "weight", "bold", NULL);  
	gtk_text_buffer_get_iter_at_line (textbuffer, &start, 0);
	gtk_text_buffer_get_iter_at_line (textbuffer, &end, 1);
	gtk_text_buffer_apply_tag (textbuffer, tag, &start, &end);
	
	gtk_text_buffer_get_iter_at_line (textbuffer, &start, 3);
	gtk_text_buffer_get_iter_at_line (textbuffer, &end, 4);
	gtk_text_buffer_apply_tag (textbuffer, tag, &start, &end);
	
	gtk_text_buffer_get_iter_at_line (textbuffer, &start, 10);
	gtk_text_buffer_get_iter_at_line (textbuffer, &end, 11);
	gtk_text_buffer_apply_tag (textbuffer, tag, &start, &end);
	
	
	gtk_dialog_run (GTK_DIALOG (aboutdialog));
	gtk_widget_destroy (aboutdialog);
}


void
on_colorselection1_color_changed       (GtkColorSelection *colorselection,
                                        gpointer         user_data)
{
	gchar spec[8];
	GdkColor color;
	gtk_color_selection_get_current_color (colorselection, &color);
	g_sprintf (spec, "#%.2x%.2x%.2x", color.red/256, color.green/256, color.blue/256);
	colorvalue = g_strdup (spec);
}


void
on_entry1_changed_event                (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkWidget *button = lookup_widget (savedialog, "save_ok");
	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (editable)));
	if (strlen (text) > 0)
	{
		gtk_widget_set_sensitive (button, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive (button, FALSE);
	}
}


void
on_list_selection_changed              (GtkTreeSelection *selection,
                                        gpointer         user_data)
{
	GtkTreeModel *model;
	gchar *color;
	gchar *type;
	GdkColor new_color, curr_color;
	GtkWidget *delete_button = lookup_widget (gcolor2, "delete_button");
	
	if (gtk_tree_selection_get_selected (selection, &model, &selection_iter))
	{
		gtk_tree_model_get (model, &selection_iter, COLOR_VALUE, &color, COLOR_TYPE, &type, -1);
		GtkColorSelection *colorsel = GTK_COLOR_SELECTION (lookup_widget (gcolor2, "colorselection1"));
		gdk_color_parse (color, &new_color);
		
		/* save the old color */
		gtk_color_selection_get_current_color (colorsel, &curr_color);
		gtk_color_selection_set_previous_color (colorsel, &curr_color);
		
		gtk_color_selection_set_current_color (colorsel, &new_color);
		g_free (color);
		
		/* if selection is a user color, enable delete button. if system color, disable (never modify the system file!) */
		if (!strcmp (type, "user"))
		{
			gtk_widget_set_sensitive (delete_button, TRUE);
		}
		else
		{
			gtk_widget_set_sensitive (delete_button, FALSE);
		}
	}
	else
	{
		/* nothing selected - disable delete button */
		gtk_widget_set_sensitive (delete_button, FALSE);
	}
}


void
on_copy_color_to_clipboard1_activate   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	
	GtkClipboard *cb = gtk_clipboard_get (gdk_atom_intern ("PRIMARY", FALSE));
	gtk_clipboard_set_text (cb, colorvalue, strlen (colorvalue));
}


void
on_show_system_colors1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)))
	{
		/* show the system colors */
		add_rgb_file (SYSTEM_FILE, "system");
	}
	else
	{
		/* hide the system colors */
		gchar *type;
		GtkTreeView *treeview = GTK_TREE_VIEW (lookup_widget (gcolor2, "treeview1"));
		GtkTreeModel *model = gtk_tree_view_get_model (treeview);
		GtkTreeIter iterator;
		
		gtk_tree_model_get_iter_first (model, &iterator);
		while (gtk_list_store_iter_is_valid (GTK_LIST_STORE (model), &iterator))
		{
			gtk_tree_model_get (model, &iterator, COLOR_TYPE, &type, -1);
			if (!strcmp (type, "system"))
			{
				gtk_list_store_remove (GTK_LIST_STORE (model), &iterator);
			}
			else
			{
				gtk_tree_model_iter_next (model, &iterator);
			}
		}

	}
}



void
show_popup_menu                        (GtkWidget       *treeview,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
					(event != NULL ? event->button : 0),
					gdk_event_get_time ((GdkEvent*)event));
}


gboolean
on_treeview1_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
	{
		GtkTreeSelection *selection;
		GtkTreePath *path;
		
		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
			event->x, event->y, &path, NULL, NULL, NULL))
		{
			gtk_tree_selection_unselect_all (selection);
			gtk_tree_selection_select_path (selection, path);
			gtk_tree_path_free (path);
		}
		show_popup_menu (widget, event, user_data);
	}
	return FALSE;
}


gboolean
on_treeview1_popup_menu                (GtkWidget       *widget,
                                        gpointer         user_data)
{
	show_popup_menu (widget, NULL, user_data);
	return TRUE;
}


void
on_save_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	savedialog = create_save_dialog ();
	gtk_window_set_transient_for (GTK_WINDOW (savedialog), GTK_WINDOW (gcolor2));
	gint result = gtk_dialog_run (GTK_DIALOG (savedialog));
	switch (result)
	{
		case GTK_RESPONSE_OK:
			save_selected_color ();
			break;
		default:
			colorname = "";
			break;
	}
	gtk_widget_destroy (savedialog);
}


void
save_selected_color                    ()
{
	GtkWidget *entry = lookup_widget (savedialog, "entry1");
	colorname = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
	add_list_color (colorvalue, colorname, "user", TRUE);
	
	GtkTreeView *list = GTK_TREE_VIEW (lookup_widget (gcolor2, "treeview1"));
	gtk_tree_view_scroll_to_cell (list, gtk_tree_path_new_first (), NULL, FALSE, 0, 0);
	gtk_tree_selection_select_path  (gtk_tree_view_get_selection (list), gtk_tree_path_new_first ());
	
	
	/* save color in user file - write at top of the file */
	FILE *fp;
	gint r, g, b;
	gchar old[512] = "";

	sscanf (colorvalue, "#%2x%2x%2x", &r, &g, &b);
	fp = fopen (get_filename (), "r");
	if (fp != NULL)
	{
		fread (old, sizeof (old), 1, fp);
		fclose (fp);
	}
	
	fp = fopen (get_filename (), "w");
	if (fp == NULL)
	{
		show_file_error ("An error occurred trying to open file \"%s\" for write access");
		return;
	}
	
	fprintf (fp, "%3d %3d %3d\t\t%s\n%s", r, g, b, colorname, old);
	fclose (fp);
}


void
on_delete_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *deletedialog = create_delete_dialog ();
	gtk_window_set_transient_for (GTK_WINDOW (deletedialog), GTK_WINDOW (gcolor2));
	gint result = gtk_dialog_run (GTK_DIALOG (deletedialog));
	switch (result)
	{
		case GTK_RESPONSE_OK:
			delete_selected_color ();
			break;
		default:
			break;
	}
	gtk_widget_destroy (deletedialog);
}


void
delete_selected_color                  ()
{
	GtkTreeView *treeview = GTK_TREE_VIEW (lookup_widget (gcolor2, "treeview1"));
	GtkTreeSelection *selection = gtk_tree_view_get_selection (treeview);
	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
	
	gchar *selection_color_name;
	gchar *selection_color_value;
	gtk_tree_model_get (model, &selection_iter, COLOR_VALUE, &selection_color_value, COLOR_NAME, &selection_color_name, -1);
	
	
	/* remove from file */
	FILE *fp;
	gchar *p;
	gchar buffer[512] = "";
	gchar newstuff[512] = "";

	fp = fopen (get_filename (), "r");
	if (fp == NULL)
	{
		show_file_error ("An error occurred trying to open file \"%s\" for read access");
		return;
	}

	gint r, g, b;
	gchar *file_color_name;
	gchar file_color_value[8];
	gboolean found = FALSE;
	while ((p = fgets (buffer, sizeof buffer, fp)) != NULL)
	{
		if (buffer[0] == '!')
			continue;
		r = g_ascii_strtoull (p, &p, 10);
		g = g_ascii_strtoull (p, &p, 10);
		b = g_ascii_strtoull (p, &p, 10);
		p += strspn (p, " \t");
		g_sprintf (file_color_value, "#%02x%02x%02x", r, g, b);
		file_color_name = g_strchomp (g_strdup (p));
		
		/* make sure to only remove the first matching color */
		if (found || strcmp (file_color_name, selection_color_name) != 0 ||
			strcmp (g_ascii_strup (file_color_value, -1), selection_color_value) != 0)
		{
			g_sprintf (newstuff, "%s%3d %3d %3d\t\t%s\n", newstuff, r, g, b, file_color_name);
		}
		else
		{
			found = TRUE;
		}
	}
	fclose (fp);
	
	fp = fopen (get_filename (), "w");
	if (fp == NULL)
	{
		show_file_error ("An error occurred trying to open file \"%s\" for write access");
		return;
	}
	
	fprintf (fp, "%s", newstuff);
	fclose (fp);
	
	
	/* remove from gui */
	gtk_list_store_remove (GTK_LIST_STORE (model), &selection_iter);
}


void
on_gcolor2_destroy                      (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit ();
}


void
on_quit_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_main_quit ();
}


