#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>

#include "interface.h"
#include "support.h"
#include "callbacks.h"

GtkListStore *liststore;
GdkWindow *gdkwin;


gchar*
get_filename()
{
	gchar buf[50];
	g_sprintf (buf, "%s/.rgb.txt", getenv ("HOME"));
	gchar* file = g_strdup (buf);
	return file;
}

void
add_list_color (gchar *spec, gchar *name, gchar *type, gboolean is_top)
{
	GdkPixmap * pixmap;
	GdkBitmap * mask;
	GtkTreeIter iter;
	gchar * swatchxpm[] =
	{
		"16 14 1 1",
		".      c #FFFFFF",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................",
		"................"
	};
	gchar colorline[] = ".      c #FFFFFF";
	
	g_sprintf (colorline, ".      c %s", g_ascii_strup (spec, -1));
	swatchxpm[1] = colorline;
	GdkPixbuf *buf = gdk_pixbuf_new_from_xpm_data ((gchar const **)swatchxpm);

	if (is_top)
		gtk_list_store_prepend (liststore, &iter);
	else
		gtk_list_store_append (liststore, &iter);
	gtk_list_store_set (liststore, &iter,
						COLOR, buf,
						COLOR_VALUE, g_ascii_strup (spec, -1),
						COLOR_NAME, name,
						COLOR_TYPE, type,
						-1);
}

void
add_rgb_file (gchar *filename, gchar *type)
{
	gchar buffer[512];
	gchar *p;
	gchar *name;
	gchar spec[8];
	gint r, g, b, lr, lg, lb;
	FILE *fp;
	
	fp = fopen (filename, "r");
	if (fp == NULL)
		return; /* silently fail */
	
	while ((p = fgets (buffer, sizeof buffer, fp)) != NULL)
	{
		if (buffer[0] == '!')
			continue;
		r = g_ascii_strtoull (p, &p, 10);
		g = g_ascii_strtoull (p, &p, 10);
		b = g_ascii_strtoull (p, &p, 10);
		p += strspn (p, " \t");
		/* X11 rgb.txt file contains duplicates; allow them in the user file though */
		if (!strcmp (type, "system") && lr == r && lg == g && lb == b)
			continue;
		else
		{
			lr = r;
			lg = g;
			lb = b;
		}
		g_sprintf (spec, "#%02x%02x%02x", r, g, b);
		name = g_strchomp (g_strdup (p));
		add_list_color (spec, name, type, FALSE);
	}
	fclose (fp);
}


gint
main (gint argc, gchar *argv[])
{

	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *treeview;
	GtkTreeSelection *select;
	
	
#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	gtk_set_locale ();
	gtk_init (&argc, &argv);
	
	add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps/" PACKAGE);

	gcolor2 = create_gcolor2 ();
	gtk_widget_show (gcolor2);
	gtk_window_set_policy (GTK_WINDOW (gcolor2), FALSE, FALSE, TRUE);
	
	
	/* setup the tree view (list) widget */
	liststore = gtk_list_store_new (N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING,
									G_TYPE_STRING, G_TYPE_STRING);
	treeview = lookup_widget (GTK_WIDGET (gcolor2), "treeview1");
	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (liststore));
	
	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (column, "Color");
	
	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_set_attributes (column, renderer, "pixbuf", COLOR, NULL);
	
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_set_attributes (column, renderer, "text", COLOR_VALUE, NULL);
	
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
	
	
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Name", renderer,
														"text", COLOR_NAME,
														NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
	
	

	/* Setup the selection handler */
	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
	g_signal_connect (G_OBJECT (select), "changed",
						G_CALLBACK (on_list_selection_changed),
						NULL);
	

	/* show first before setting gdkwin */
	gtk_widget_show (gcolor2);
	gdkwin = GTK_WIDGET (gcolor2)->window;
	
	
	/* fill with user-saved values */
	add_rgb_file (get_filename (), "user");
	add_rgb_file (SYSTEM_FILE, "system");
	
	
	/* Setup the popup menu */
	menu = create_menu1 ();
	
	gtk_main ();
	return 0;
}

